package cn.com.qmth.stmms.admin.thread;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.report.service.ReportService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectClassService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectCollegeService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectGroupService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectQuestionService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectService;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectTeacherService;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.MarkStatus;

/**
 * 统计定时任务
 *
 */
@Component
public class ScoreReportJob {

    protected static final Logger log = LoggerFactory.getLogger(ScoreReportJob.class);

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private ExamService examService;

    @Autowired
    private FileService fileService;

    @Autowired
    private LockService lockService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ReportService reportService;

    @Autowired
    private ReportSubjectQuestionService reportSubjectQuestionService;

    @Autowired
    private ReportSubjectClassService reportSubjectClassService;

    @Autowired
    private ReportSubjectTeacherService reportSubjectTeacherService;

    @Autowired
    private ReportSubjectCollegeService reportSubjectCollegeService;

    @Autowired
    private ReportSubjectGroupService reportSubjectGroupService;

    @Autowired
    private ReportSubjectService reportSubjectService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    /**
     * 定时更新成绩分析报告
     */
    @Scheduled(fixedDelay = 30 * 60 * 1000, initialDelay = 2 * 60 * 1000)
    public void updateReport() {
        log.info("start auto-update report");
        try {
            Map<Integer, Set<String>> map = new HashMap<Integer, Set<String>>();
            List<Integer> examIds = groupService.findExamIdByStatus(MarkStatus.FORMAL);
            for (Integer examId : examIds) {
                List<ExamSubject> list = subjectService.list(examId);
                Set<String> set = new HashSet<>();
                for (ExamSubject subject : list) {
                    List<MarkGroup> groups = groupService.findByExamAndSubjectAndStatus(subject.getExamId(),
                            subject.getCode(), MarkStatus.TRIAL);
                    if (!groups.isEmpty()
                            && !lockService.isLocked(LockType.SCORE_CALCULATE, subject.getExamId(), subject.getCode())) {
                        set.add(subject.getCode());
                    }
                }
                if (!set.isEmpty()) {
                    map.put(examId, set);
                }
            }
            ScoreReportThread thread = new ScoreReportThread(map, lockService, studentService, questionService,
                    groupService, reportService, examService, subjectService, fileService, false,
                    reportSubjectQuestionService, reportSubjectClassService, reportSubjectTeacherService,
                    reportSubjectCollegeService, reportSubjectGroupService, reportSubjectService);
            taskExecutor.submit(thread);
        } catch (Exception e) {
            log.error("auto-update report error", e);
        } finally {
            log.info("finish auto-update report");
        }
    }
}
